﻿using ProjectPublishing.Entities.Enums;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPublishing.Entities
{
    public class Order
    {
        public int Id { get; private set; }

        [DisplayName("Дата заказа")]
        public DateTime OrderDate { get; private set; }

        [DisplayName("Описание")]
        public string Description { get; private set; } = string.Empty;

        [DisplayName("Тип продукта")]
        public ProductType ProductType { get; private set; }

        [DisplayName("Количество")]
        public int Amount { get; private set; }

        [DisplayName("Статус заказа")]
        public OrderStatus Status { get; private set; }

        [Browsable(false)]
        public int CustomerId { get; private set; }

        [DisplayName("Имя покупателя")]
        public string CustomerName { get; private set; } = string.Empty;

        [Browsable(false)]
        public int PrintingId { get; private set; }

        [DisplayName("Название типографии")]
        public string PrintingName { get; private set; } = string.Empty;

        [Browsable(false)]
        public IEnumerable<OrderMaterials> Materials { get; private set; } = [];

        public string MaterialsList => Materials != null ? string.Join(", ", Materials.Select(x => $"{ x.MaterialsId }")) : string.Empty; // !!!отображение названия!!!

        public static Order CreateOrder(int id, string description, ProductType productType, int amount, OrderStatus status, int customerId, int printingId, IEnumerable<OrderMaterials> materials)
        {
            return new Order { ProductType = productType, Id = id, Amount = amount, Status = status, CustomerId = customerId, PrintingId = printingId, Materials = materials, OrderDate = DateTime.Now, Description = description ?? string.Empty };
        }

        public static Order CreateOrder(TempOrderMaterials tempOrderMaterials, IEnumerable<OrderMaterials> materials)
        {
            return new Order 
            { 
                ProductType = tempOrderMaterials.ProductType, 
                Id = tempOrderMaterials.Id, 
                Amount = tempOrderMaterials.Amount, 
                Status = tempOrderMaterials.Status, 
                CustomerId = tempOrderMaterials.CustomerId, 
                PrintingId = tempOrderMaterials.PrintingId, 
                Materials = materials, 
                OrderDate = DateTime.Now, 
                Description = tempOrderMaterials.Description ?? string.Empty };
        }
    }
}
