﻿using ProjectPublishing.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Unity;

namespace ProjectPublishing.Forms
{
    public partial class FormContactPeople : Form
    {
        private readonly IUnityContainer _container;
        private readonly IContactPersonRepository _contactPersonRepository;
        public FormContactPeople(IUnityContainer container, IContactPersonRepository contactPersonRepository)
        {
            InitializeComponent();
            _container = container ?? throw new ArgumentException(nameof(container));
            _contactPersonRepository = contactPersonRepository ?? throw new ArgumentException(nameof(contactPersonRepository));
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            try
            {
                _container.Resolve<FormContactPerson>().ShowDialog();
                loadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при добавлении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonEdit_Click(object sender, EventArgs e)
        {
            if (!tryGetIDFromRow(out var findId))
            {
                return;
            }
            try
            {
                var form = _container.Resolve<FormContactPerson>();
                form.Id = findId;
                form.ShowDialog();
                loadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при изменении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonRemove_Click(object sender, EventArgs e)
        {
            if (!tryGetIDFromRow(out var findId))
            {
                return;
            }
            if (MessageBox.Show("Удалить запись?", "Удаление", MessageBoxButtons.YesNo) != DialogResult.Yes)
            {
                return;
            }

            try
            {
                _contactPersonRepository.DeleteContactPerson(findId);
                loadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при удалении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void FormContactPeople_Load(object sender, EventArgs e)
        {
            try
            {
                loadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при загрузке", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void loadList()
        {
            dataGridView.DataSource = _contactPersonRepository.ReadContactPeople();
            dataGridView.Columns["Id"].Visible = false;
        }

        private bool tryGetIDFromRow(out int id)
        {
            id = 0;
            if (dataGridView.SelectedRows.Count < 1)
            {
                MessageBox.Show("Нет выбранной записи", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
            return true;
        }
    }
}
