﻿using ProjectPublishing.Entities;
using ProjectPublishing.Repositories;
using ProjectPublishing.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectPublishing.Forms
{
    public partial class FormContactPerson : Form
    {
        private readonly IContactPersonRepository _contactPersonRepository;
        private int? _contactPersonId;

        public int Id
        {
            set
            {
                try
                {
                    var contact = _contactPersonRepository.ReadContactPersonById(value);
                    if (contact == null)
                    {
                        throw new InvalidDataException(nameof(contact));
                    }
                    textBoxPhone.Text = contact.PhoneNumber;
                    textBoxEmail.Text = contact.Email;
                    textBoxName.Text = contact.Name;
                    _contactPersonId = value;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
        }
        public FormContactPerson(IContactPersonRepository contactPersonRepository)
        {
            InitializeComponent();
            _contactPersonRepository = contactPersonRepository ?? throw new ArgumentException(nameof(contactPersonRepository));
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxPhone.Text) || string.IsNullOrWhiteSpace(textBoxEmail.Text) || string.IsNullOrWhiteSpace(textBoxName.Text))
                {
                    throw new Exception("Имеются незаполненные поля!");
                }

                if (_contactPersonId.HasValue)
                {
                    _contactPersonRepository.UpdateContactPerson(CreateContactPerson(_contactPersonId.Value));
                }
                else
                {
                    _contactPersonRepository.CreateContactPerson(CreateContactPerson(0));
                }
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e) => Close();

        private ContactPerson CreateContactPerson(int id) => ContactPerson.CreateContactPerson(id, textBoxName.Text, textBoxPhone.Text, textBoxEmail.Text);
    }
}
