﻿using ProjectPublishing.Entities;
using ProjectPublishing.Entities.Enums;
using ProjectPublishing.Repositories;
using ProjectPublishing.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectPublishing.Forms
{
    public partial class FormMaterial : Form
    {
        private readonly IMaterialRepository _materialRepository;
        private int? _materialId;

        public int Id
        {
            set
            {
                try
                {
                    var material = _materialRepository.ReadMaterialById(value);
                    if (material == null)
                    {
                        throw new InvalidDataException(nameof(material));
                    }
                    textBoxName.Text = material.Name;
                    comboBoxMaterialType.SelectedItem = material.MaterialType;
                    _materialId = value;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
        }

        public FormMaterial(IMaterialRepository materialRepository)
        {
            InitializeComponent();
            _materialRepository = materialRepository ?? throw new ArgumentException(nameof(materialRepository));
            comboBoxMaterialType.DataSource = Enum.GetValues(typeof(MaterialType));
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxName.Text))
                {
                    throw new Exception("Имеются незаполненные поля!");
                }
                if (_materialId.HasValue)
                {
                    _materialRepository.UpdateMaterial(CreateMaterial(_materialId.Value));
                }
                else
                {
                    _materialRepository.CreateMaterial(CreateMaterial(0));
                }
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e) => Close();

        private Material CreateMaterial(int id) => Material.CreateMaterial(id, textBoxName.Text, (MaterialType)comboBoxMaterialType.SelectedItem!);
    }
}
