﻿using ProjectPublishing.Reports;
using ProjectPublishing.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Unity;

namespace ProjectPublishing.Forms
{
    public partial class FormMaterialsReport : Form
    {
        private string _fileName = string.Empty;
        private readonly IUnityContainer _container;
        public FormMaterialsReport(IUnityContainer container, IMaterialRepository materialRepository)
        {
            InitializeComponent();
            _container = container ?? throw new ArgumentNullException(nameof(container));

            comboBoxMaterials.DataSource = materialRepository.ReadMaterials();
            comboBoxMaterials.DisplayMember = "Name";
            comboBoxMaterials.ValueMember = "Id";
        }

        private void ButtonSelectFilePath_Click(object sender, EventArgs e)
        {
            var sfd = new SaveFileDialog()
            {
                Filter = "Excel Files | *.xlsx"
            };

            if (sfd.ShowDialog() != DialogResult.OK)
                return;

            textBoxFilePath.Text = sfd.FileName;
        }

        private void ButtonMakeReport_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxFilePath.Text))
                    throw new Exception("Отсутствует имя файла для отчета");

                if (comboBoxMaterials.SelectedIndex < 0 || comboBoxMaterials.SelectedIndex < 0)
                    throw new Exception("Не выбран материал");

                if (dateTimePickerEnd.Value <= dateTimePickerStart.Value)
                    throw new Exception("Дата начала должна быть раньше даты окончания");

                if (_container.Resolve<TableReport>().CreateTable(textBoxFilePath.Text, (int)comboBoxMaterials.SelectedValue!, dateTimePickerStart.Value, dateTimePickerEnd.Value))
                    MessageBox.Show("Документ сформирован", "Формирование документа", MessageBoxButtons.OK, MessageBoxIcon.Information);
                else
                    MessageBox.Show("Возникли ошибки при формировании документа.Подробности в логах", "Формирование документа", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при создании очета", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
