﻿using ProjectPublishing.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Unity;

namespace ProjectPublishing.Forms
{
    public partial class FormOrders : Form
    {
        private readonly IUnityContainer _container;
        private readonly IOrderRepository _orderRepository;
        public FormOrders(IUnityContainer container, IOrderRepository orderRepository)
        {
            InitializeComponent();
            _container = container ?? throw new ArgumentException(nameof(container));
            _orderRepository = orderRepository ?? throw new ArgumentException(nameof(orderRepository));
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            try
            {
                _container.Resolve<FormOrder>().ShowDialog();
                loadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при добавлении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonRemove_Click(object sender, EventArgs e)
        {
            if (!tryGetIDFromRow(out var findId))
            {
                return;
            }
            if (MessageBox.Show("Удалить запись?", "Удаление", MessageBoxButtons.YesNo) != DialogResult.Yes)
            {
                return;
            }

            try
            {
                _orderRepository.DeleteOrder(findId);
                loadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при удалении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void FormOrders_Load(object sender, EventArgs e)
        {
            try
            {
                loadList();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при загрузке", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void loadList()
        {
            dataGridView.DataSource = _orderRepository.ReadOrders();
            dataGridView.Columns["Id"].Visible = false;
        }

        private bool tryGetIDFromRow(out int id)
        {
            id = 0;
            if (dataGridView.SelectedRows.Count < 1)
            {
                MessageBox.Show("Нет выбранной записи", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
            return true;
        }
    }
}
