﻿using Microsoft.Extensions.Logging;
using ProjectPublishing.Repositories;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPublishing.Reports
{
    public class ChartReport
    {
        private readonly IMaterialAdmissionRepository _materialRepository;

        private readonly ILogger<ChartReport> _logger;

        public ChartReport(IMaterialAdmissionRepository materialRepository,
            ILogger<ChartReport> logger)
        {
            _materialRepository = materialRepository ??
                throw new ArgumentNullException(nameof(materialRepository));
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
        }

        public bool CreateChart(string filePath, DateTime dateTime)
        {
            try
            {
                new PdfBuilder(filePath)
                    .AddHeader("Распределение Материалов при поступлении")
                    .AddPieChart("Материалы", GetData(dateTime))
                    .Build();

                return true;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при формировании документа");
                return false;
            }
        }

        private List<(string Caption, double Value)> GetData(DateTime dateTime)
        {
            return _materialRepository
                .ReadMaterialsAdmission(endDate: dateTime)
                .GroupBy(x => x.MaterialName, (key, group) => new
                {
                    Id = key,
                    Count = group.Sum(x => x.Amount)
                })
                .Select(x => (x.Id.ToString(), (double)x.Count))
                .ToList();
        }
    }
}