﻿using Microsoft.Extensions.Logging;
using ProjectPublishing.Entities;
using ProjectPublishing.Repositories;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPublishing.Reports
{
    public class DocReport
    {
        private readonly ICustomerRepository _customerRepository;
        private readonly IPrintingHouseRepository _printingHouseRepository;
        private readonly IMaterialRepository _materialRepository;
        private readonly IContactPersonRepository _contactPersonRepository;
        private readonly ILogger<DocReport> _logger;
        public DocReport(ICustomerRepository customerRepository, IPrintingHouseRepository printingHouseRepository, IMaterialRepository materialRepository, IContactPersonRepository contactPersonRepository, ILogger<DocReport> logger) 
        {
            _customerRepository = customerRepository ?? throw new ArgumentNullException(nameof(customerRepository));
            _printingHouseRepository = printingHouseRepository ?? throw new ArgumentNullException(nameof(printingHouseRepository));
            _materialRepository = materialRepository ?? throw new ArgumentNullException(nameof(materialRepository));
            _contactPersonRepository = contactPersonRepository ?? throw new ArgumentNullException(nameof(contactPersonRepository));
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
        }

        public bool CreateDoc(string filePath, bool includeCustomer, bool includePrintingHouse, bool includeMaterial, bool includeContactPerson)
        {
            try
            {
                var builder = new WordBuilder(filePath).AddHeader("Документ со справочниками");
                if (includeCustomer)
                    builder.AddParagraph("Заказчики").AddTable([2400, 2400, 1200], GetCustomers());

                if (includePrintingHouse)
                    builder.AddParagraph("Типографии").AddTable([2400, 2400, 2400, 2400], GetPrintingHouses());

                if (includeMaterial)
                    builder.AddParagraph("Материалы").AddTable([2400, 1200], GetMaterials());

                if (includeContactPerson)
                    builder.AddParagraph("Контактные личности").AddTable([2400, 2400, 1200], GetContactPeople());

                builder.Build();

                return true;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при формировании документа");
                return false;
            }
        }

        private List<string[]> GetCustomers()
        {
            return [
            ["Номер телефона", "Email", "ID типографии"],
            .. _customerRepository
                .ReadCustomers()
                .Select(x => new string[] { x.PhoneNumber, x.Email, x.PublisherId.ToString() }),
            ];
        }

        private List<string[]> GetPrintingHouses()
        {
            return [
            ["Название", "Адрес", "Номер телефона", "Email"],
            .. _printingHouseRepository
                .ReadPrintingHouses()
                .Select(x => new string[] {x.Name, x.Address, x.PhoneNumber, x.Email }),
            ];
        }

        private List<string[]> GetMaterials()
        {
            return [
            ["Название", "Тип материала"],
            .. _materialRepository
                .ReadMaterials()
                .Select(x => new string[] {x.Name, x.MaterialType.ToString() }),
            ];
        }

        private List<string[]> GetContactPeople()
        {
            return [
            ["Имя", "Номер телефона", "Email"],
            .. _contactPersonRepository
                .ReadContactPeople()
                .Select(x => new string[] {x.Name, x.PhoneNumber, x.Email }),
            ];
        }
    }
}
