﻿using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using ProjectPublishing.Repositories;
using ProjectPublishing.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPublishing.Reports
{
    public class TableReport
    {
        private readonly IOrderRepository _orderRepository;
        private readonly IMaterialAdmissionRepository _materialAdmissionRepository;
        
        private readonly ILogger<TableReport> _logger;
        internal static readonly string[] item = ["Материал", "Дата", "Количество поступило", "Количество закупок"];
        public TableReport(IOrderRepository orderRepository, IMaterialAdmissionRepository materialAdmissionRepository, ILogger<TableReport> logger)
        {
            _orderRepository = orderRepository ?? throw new ArgumentNullException(nameof(orderRepository));
            _materialAdmissionRepository = materialAdmissionRepository ?? throw new ArgumentNullException(nameof(materialAdmissionRepository));
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
        }

        public bool CreateTable(string filePath, int materialId, DateTime startDate, DateTime endDate)
        {
            try
            {
                new ExcelBuilder(filePath)
                    .AddHeader("Сводка по материалам", 0, 4)
                    .AddParagraph($"за период с {startDate:dd.MM.yyyy} по {endDate:dd.MM.yyyy}", 0)
                    .AddTable([15, 15, 15, 15], GetData(materialId, startDate, endDate))
                    .Build();
                return true;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при формировании документа");
                return false;
            }
        }

        private List<string[]> GetData(int materialId, DateTime startDate, DateTime endDate)
        {
            var data = _materialAdmissionRepository
                .ReadMaterialsAdmission(startDate, endDate, materialId)
                .Select(x => new {
                    MaterialId = x.MaterialId,
                    MaterialName = x.MaterialName,
                    Date = x.DateTime,
                    CountIn = (int?)x.Amount,
                    CountOut = (int?)1
                }).
                ToList();

            //var data = _materialAdmissionRepository
            //    .ReadMaterialsAdmission(startDate, endDate, materialId)
            //    .Select(x => new {
            //        MaterialId= x.MaterialId,
            //        MaterialName = x.MaterialName,
            //        Date = x.DateTime,
            //        CountIn = (int?)x.Amount,
            //        CountOut = (int?)null
            //    })
            //    .Union(
            //    _orderRepository
            //    .ReadOrders() // тут где-то
            //    .Where(x => x.OrderDate >= startDate && x.OrderDate <= endDate.AddDays(1) && x.Materials.Any(y => y.MaterialsId == materialId))
            //    .SelectMany(order => order.Materials
            //    .Where(m => m.MaterialsId == materialId)
            //        .Select(m => new
            //        {
            //            MaterialId = m.MaterialsId,
            //            Date = order.OrderDate,
            //            CountIn = (int?)null,
            //            CountOut = (int?)m.Amount
            //        })))
            //.OrderBy(x => x.Date);
            _logger.LogDebug("Объединенные данные в отчет: {json}", JsonConvert.SerializeObject(data));
            return new List<string[]> { item }
                .Union(
                    data.Select(x => new string[] {
                    x.MaterialId.ToString(),
                    x.Date.ToString("dd.MM.yyyy"),
                    x.CountIn?.ToString() ?? string.Empty,
                    x.CountOut?.ToString() ?? string.Empty
                    }))
                .Union(new[] {
                new string[] {
                    "Всего",
                    string.Empty,
                    data.Sum(x => x.CountIn ?? 0).ToString(),
                    data.Sum(x => x.CountOut ?? 0).ToString()
                }
                })
                .ToList();
        }
    }
}
