﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectPublishing.Entities;
using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Text.Json.Serialization;
using System.Threading.Tasks;

namespace ProjectPublishing.Repositories.Implementations
{
    public class ContactPersonRepository : IContactPersonRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<ContactPersonRepository> _logger;
        public ContactPersonRepository(IConnectionString connectionString, ILogger<ContactPersonRepository> logger) 
        {
            _connectionString = connectionString;
            _logger = logger;
        }
        public void CreateContactPerson(ContactPerson contactPerson)
        {
            _logger.LogInformation("Создание объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(contactPerson));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryInsert = @"INSERT INTO ContactPersonPublisher (Name, PhoneNumber, Email)
VALUES (@Name, @PhoneNumber, @Email)";
                connection.Execute(queryInsert, contactPerson);
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }

        public void UpdateContactPerson(ContactPerson contactPerson)
        {
            _logger.LogInformation("Редактирование объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(contactPerson));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryUpdate = @"
UPDATE ContactPersonPublisher
SET 
Name=@Name, 
PhoneNumber=@PhoneNumber, 
Email=@Email
WHERE Id=@Id";
                connection.Execute(queryUpdate, contactPerson);
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при редактировании объекта");
                throw;
            }
        }
        public void DeleteContactPerson(int id)
        {
            _logger.LogInformation("Удаление объекта");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryDelete = @"
DELETE FROM ContactPersonPublisher
WHERE Id=@id";
                connection.Execute(queryDelete, new { id });
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении объекта");
                throw;
            }
        }

        public IEnumerable<ContactPerson> ReadContactPeople()
        {
            _logger.LogInformation("Получение всех объектов");
            
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = "SELECT * FROM ContactPersonPublisher";
                var contactPeople = connection.Query<ContactPerson>(querySelect);
                _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(contactPeople));
                return contactPeople;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении объектов");
                throw;
            }
        }

        public ContactPerson ReadContactPersonById(int id)
        {
            _logger.LogInformation("Получение объекта по ID");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var querySelect = @"
SELECT * FROM ContactPersonPublisher
WHERE Id=@id";
                var contactPerson = connection.QueryFirst<ContactPerson>(querySelect, new {id});
                connection.Close();
                _logger.LogDebug("Найден объект: {json}", JsonConvert.SerializeObject(contactPerson));
                return contactPerson;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при поиске объекта");
                throw;
            }
        }

        
    }
}
