﻿using Dapper;
using DocumentFormat.OpenXml.Drawing.Charts;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectPublishing.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace ProjectPublishing.Repositories.Implementations
{
    public class MaterialAdmissionRepository : IMaterialAdmissionRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<MaterialAdmissionRepository> _logger;
        public MaterialAdmissionRepository(IConnectionString connectionString, ILogger<MaterialAdmissionRepository> logger) 
        {
            _connectionString = connectionString;
            _logger = logger;
        }
        public void CreateMaterialsAdmission(MaterialsAdmission materialAdmission)
        {
            _logger.LogInformation("Создание объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(materialAdmission));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryInsert = @"
INSERT INTO MaterialAdmission (Amount, MaterialId)
VALUES (@Amount, @MaterialId)";
                connection.Execute(queryInsert, materialAdmission);
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }

        public IEnumerable<MaterialsAdmission> ReadMaterialsAdmission(DateTime? startDate = null, DateTime? endDate = null, int? materialId = null)
        {
            _logger.LogInformation("Получение всех объектов");

            try
            {
                var builder = new QueryBuilder();
                if (startDate.HasValue)
                {
                    builder.AddCondition("ma.dateTime >= @startDate");
                }
                if (endDate.HasValue)
                {
                    builder.AddCondition("ma.dateTime <= @endDate");
                }
                if (materialId.HasValue)
                {
                    builder.AddCondition("ma.MaterialId = @materialId");
                }
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = @$"SELECT ma.*, m.Name as MaterialName FROM MaterialAdmission as ma LEFT JOIN Material as m ON m.Id = ma.materialId {builder.Build()}";
                var materialAdmissions = connection.Query<MaterialsAdmission>(querySelect, new {startDate, endDate, materialId});
                _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(materialAdmissions));
                return materialAdmissions;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении объектов");
                throw;
            }
        }

    }
}
