﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectPublishing.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPublishing.Repositories.Implementations
{
    public class MaterialRepository : IMaterialRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<MaterialRepository> _logger;

        public MaterialRepository(IConnectionString connectionString, ILogger<MaterialRepository> logger)
        {
            _connectionString = connectionString;
            _logger = logger;
        }

        public void CreateMaterial(Material material)
        {
            _logger.LogInformation("Создание объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(material));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryInsert = @"INSERT INTO Material (Name, MaterialType)
VALUES (@Name, @MaterialType)";
                connection.Execute(queryInsert, material);
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }

        public void DeleteMaterial(int id)
        {
            _logger.LogInformation("Удаление объекта");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryDelete = @"
DELETE FROM Material
WHERE Id=@id";
                connection.Execute(queryDelete, new { id });
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении объекта");
                throw;
            }
        }

        public void UpdateMaterial(Material material)
        {
            _logger.LogInformation("Редактирование объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(material));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryUpdate = @"
UPDATE Material
SET 
Name=@Name, 
MaterialType=@MaterialType
WHERE Id=@Id";
                connection.Execute(queryUpdate, material);
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при редактировании объекта");
                throw;
            }
        }

        public Material ReadMaterialById(int id)
        {
            _logger.LogInformation("Получение объекта по ID");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var querySelect = @"
SELECT * FROM Material
WHERE Id=@id";
                var material = connection.QueryFirst<Material>(querySelect, new { id });
                connection.Close();
                _logger.LogDebug("Найден объект: {json}", JsonConvert.SerializeObject(material));
                return material;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при поиске объекта");
                throw;
            }
        }

        public IEnumerable<Material> ReadMaterials()
        {
            _logger.LogInformation("Получение всех объектов");

            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = "SELECT * FROM Material";
                var contactPeople = connection.Query<Material>(querySelect);
                _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(contactPeople));
                return contactPeople;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении объектов");
                throw;
            }
        }

        
    }
}
