﻿using Dapper;
using DocumentFormat.OpenXml.Office2010.Excel;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectPublishing.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPublishing.Repositories.Implementations
{
    public class OrderMaterialsRepository : IOrderMaterialsRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<OrderMaterialsRepository> _logger;

        public OrderMaterialsRepository(ILogger<OrderMaterialsRepository> logger, IConnectionString connectionString)
        {
            _logger = logger;
            _connectionString = connectionString;
        }

        public IEnumerable<OrderMaterials> ReadOrderMaterials(DateTime? startDate = null, DateTime? endDate = null, int? materialId = null)
        {
            _logger.LogInformation("Получение всех объектов");

            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = "SELECT * FROM OrderMaterials";
                var contactPeople = connection.Query<OrderMaterials>(querySelect);
                _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(contactPeople));
                return contactPeople;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении объектов");
                throw;
            }
        }

        public OrderMaterials ReadOrderMaterialsByIds(int orderId, int materialId)
        {
            _logger.LogInformation("Получение объекта по ID");
            _logger.LogDebug("Объект: {orderId}, {materialId}", orderId, materialId);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var querySelect = @"
SELECT * FROM OrderMaterials
WHERE OrderId=@orderId AND MaterialId = @materialId";
                var orderMaterial = connection.QueryFirst<OrderMaterials>(querySelect, new { orderId, materialId });
                connection.Close();
                _logger.LogDebug("Найден объект: {json}", JsonConvert.SerializeObject(orderMaterial));
                return orderMaterial;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при поиске объекта");
                throw;
            }
        }
    }
}
