﻿using Dapper;
using DocumentFormat.OpenXml.Drawing;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectPublishing.Entities;
using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPublishing.Repositories.Implementations
{
    public class OrderRepository : IOrderRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<OrderRepository> _logger;
        public OrderRepository(IConnectionString connectionString, ILogger<OrderRepository> logger)
        {
            _connectionString = connectionString;
            _logger = logger;
        }

        public void CreateOrder(Order order)
        {
            _logger.LogInformation("Создание объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(order));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                using var transaction = connection.BeginTransaction();
                
                var queryInsert = @"
INSERT INTO PrintingOrder (Description, ProductType, Amount, Status, CustomerId, PrintingId)
VALUES (@Description, @ProductType, @Amount, @Status, @CustomerId, @PrintingId);
SELECT MAX(Id) FROM PrintingOrder";
                var PrintingOrderId = connection.QueryFirst<int>(queryInsert, order, transaction);

                var querySubInsert = @"
INSERT INTO OrderMaterials (MaterialId, OrderId, Amount)
VALUES (@MaterialsId, @OrderId, @Amount)";
                foreach (var elem in order.Materials)
                {
                    _logger.LogDebug("Проверка MaterialId: {MaterialId}", elem.MaterialsId);
                    connection.Execute(querySubInsert, new { elem.MaterialsId, OrderId = PrintingOrderId, elem.Amount }, transaction);
                }
                transaction.Commit();
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }

        public void DeleteOrder(int id)
        {
            _logger.LogInformation("Удаление объекта");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryDelete = @"
DELETE FROM OrderMaterials
WHERE OrderId=@id;
DELETE FROM PrintingOrder
WHERE Id=@id";
                connection.Execute(queryDelete, new { id });
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении объекта");
                throw;
            }
        }

        public IEnumerable<Order> ReadOrders()
        {
            _logger.LogInformation("Получение всех объектов");

            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = @"
SELECT o.*,
ma.MaterialId as MaterialsId, 
ma.Amount as MaterialsAmount,
CONCAT(c.LastName, ' ', c.FirstName) as CustomerName,
ph.Name as PrintingName
FROM PrintingOrder o
LEFT JOIN OrderMaterials ma ON o.Id = ma.OrderId
LEFT JOIN Customer c ON o.CustomerId = c.Id
LEFT JOIN PrintingHouse ph ON o.PrintingId = ph.Id
"; // возможно сюда фикс
                var printingOrders = connection.Query<TempOrderMaterials>(querySelect);
                _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(printingOrders));

                return printingOrders.GroupBy(x => x.Id, y => y, (key, value) =>
                Order.CreateOrder(value.First(), value.Select(z => OrderMaterials.Create(0, z.MaterialsId, z.MaterialsAmount)))).ToList();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении объектов");
                throw;
            }
        }
    }
}
