﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectPublishing.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPublishing.Repositories.Implementations
{
    public class PrintingHouseRepository : IPrintingHouseRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<PrintingHouseRepository> _logger;

        public PrintingHouseRepository(IConnectionString connectionString, ILogger<PrintingHouseRepository> logger)
        {
            _connectionString = connectionString;
            _logger = logger;
        }
        public void CreatePrintingHouse(PrintingHouse printingHouse)
        {
            _logger.LogInformation("Создание объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(printingHouse));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryInsert = @"
INSERT INTO PrintingHouse (Name, Address, PhoneNumber, Email)
VALUES (@Name, @Address, @PhoneNumber, @Email)";
                connection.Execute(queryInsert, printingHouse);
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }

        public void UpdatePrintingHouse(PrintingHouse printingHouse)
        {
            _logger.LogInformation("Редактирование объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(printingHouse));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryUpdate = @"
UPDATE PrintingHouse
SET
Name=@Name,
Address=@Address,
PhoneNumber=@PhoneNumber, 
Email=@Email
WHERE Id=@Id";
                connection.Execute(queryUpdate, printingHouse);
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при редактировании объекта");
                throw;
            }
        }

        public void DeletePrintingHouse(int id)
        {
            _logger.LogInformation("Удаление объекта");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var queryDelete = @"
DELETE FROM PrintingHouse
WHERE Id=@id";
                connection.Execute(queryDelete, new { id });
                connection.Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении объекта");
                throw;
            }
        }

        public PrintingHouse ReadPrintingHouseById(int id)
        {
            _logger.LogInformation("Получение объекта по ID");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                connection.Open();
                var querySelect = @"
SELECT * FROM PrintingHouse
WHERE Id=@id";
                var printingHouse = connection.QueryFirst<PrintingHouse>(querySelect, new { id });
                connection.Close();
                _logger.LogDebug("Найден объект: {json}", JsonConvert.SerializeObject(printingHouse));
                return printingHouse;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при поиске объекта");
                throw;
            }
        }

        public IEnumerable<PrintingHouse> ReadPrintingHouses()
        {
            _logger.LogInformation("Получение всех объектов");

            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = "SELECT * FROM PrintingHouse";
                var printingHouse = connection.Query<PrintingHouse>(querySelect);
                _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(printingHouse));
                return printingHouse;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении объектов");
                throw;
            }
        }

    }
}
